/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2001 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: David Eriksson <eriksson@php.net>                            |
   +----------------------------------------------------------------------+
 */

#include "tocorba.h"
#include "zppvalue.h"
#include "zppexception.h"
#include "object.h"
#include "struct.h"

/**
	Convert a sequence
*/
static void to_corba_sequence(ZppValue& rSource, CORBA::Any& rDestination, CORBA::TypeCode_ptr pType)
{
	Assert(rSource.IsArray(), ZppTypeMismatchException_());

	CORBA::TypeCode_var content_type = pType->content_type();
	
	const ZvalHashTable& source = rSource.HashTable();
	const unsigned size = source.Size();
	rDestination.seq_put_begin(size);

	for (unsigned i = 0; i < size; i++)
	{
		ZppValue element(source[i]);
		to_corba(element, rDestination, content_type);
	}

	rDestination.seq_put_end();
}

void to_corba(ZppValue& rSource, CORBA::Any& rDestination, CORBA::TypeCode_ptr pType)
{
	if (NULL == pType)
		pType = rDestination.type(); 

	switch (pType->kind())
	{
		case CORBA::tk_short:	
			rSource.ToLong(); 
			rDestination <<= (CORBA::Short)rSource.Long();
			break;
		case CORBA::tk_ushort:
			rSource.ToLong(); 
			rDestination <<= (CORBA::UShort)rSource.Long();
			break;
		case CORBA::tk_long: 
			rSource.ToLong(); 
			rDestination <<= (CORBA::Long)rSource.Long();
			break;
		case CORBA::tk_ulong:	
			rSource.ToLong(); 
			rDestination <<= (CORBA::ULong)rSource.Long();
			break;
													
		case CORBA::tk_float:
			rSource.ToDouble(); 
			rDestination <<= (CORBA::Float)rSource.Double();
			break;
			
		case CORBA::tk_double: 	
			rSource.ToDouble();
			rDestination <<= (CORBA::Double)rSource.Double();
			break;

		case CORBA::tk_boolean:
			rSource.ToBool();
			rDestination <<= CORBA::Any::from_boolean(rSource.Bool());
			break;
														
		case CORBA::tk_char:
			if (rSource.IsString())
			{
				// to allow assignment like this: $x = 'X';
				rDestination <<= CORBA::Any::from_char(rSource.String()[0]);
			}
			else
			{
				rSource.ToLong();
				rDestination <<= CORBA::Any::from_char(rSource.Long());
			}
			break;
		case CORBA::tk_octet: 
			rSource.ToLong();
			rDestination <<= CORBA::Any::from_octet(rSource.Long());
			break;
		
		case CORBA::tk_string: 
			if (rSource.IsNull())
			{
				rDestination <<= CORBA::Any::from_string("", pType->length());
			}
			else
			{
				rSource.ToString();
				rDestination <<= CORBA::Any::from_string(rSource.String(), pType->length());
			}
			break;

		case CORBA::tk_enum: 
			// Uses mico-extension TypeCode::enum_put
			if (rSource.IsString())
			{
				rDestination.enum_put(pType->member_index(rSource.String()));	
			}
			else
			{
				rSource.ToLong();
				// TODO: validate range
				rDestination.enum_put(rSource.Long());
			}
			break;
		
		case CORBA::tk_struct:
		case CORBA::tk_except: 
			UniverseStruct::ConvertToCorba(rSource, rDestination, pType);
			break;
			
		case CORBA::tk_objref: 
			UniverseObject::ConvertToCorba(rSource, rDestination, pType);
			break;
			
		case CORBA::tk_sequence: 
			to_corba_sequence(rSource, rDestination, pType);
			break;

		case CORBA::tk_alias: 
			// Use very nice MICO extension: TypeCode::unalias()
			to_corba(rSource, rDestination, pType->unalias());
			break;
			
		case CORBA::tk_any: 
		case CORBA::tk_TypeCode: 
		case CORBA::tk_Principal: 
		case CORBA::tk_union: 
		case CORBA::tk_array: 
		case CORBA::tk_longlong: 
		case CORBA::tk_ulonglong: 
		case CORBA::tk_longdouble: 
		case CORBA::tk_wchar: 
		case CORBA::tk_wstring: 
		case CORBA::tk_fixed: 
		case CORBA::tk_value: 
		case CORBA::tk_value_box: 
		case CORBA::tk_native: 
		case CORBA::tk_abstract_interface:
#if MICO_BIN_VERSION >= 0x020306
		case CORBA::tk_local_interface:
#endif
		case CORBA::tk_null:
		case CORBA::tk_void: 
		default:
			{
				ostrstream_zpp msg;
				msg << "Conversion to this CORBA data type (" << pType->kind() << ") is not yet supported" << ends;
				throw ZppError_(msg.str());
			}
	}
	
}

